window.Gleam.initSearch([{"doc":"gs","title":"gs","content":"# gs - Gleam Streams\n\n[![Package Version](https://img.shields.io/hexpm/v/gs)](https://hex.pm/packages/gs)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/gs/)\n\n`gs` - A lightweight Gleam library that introduces the `Stream` type for handling lazy value sequences. The library offers comprehensive functionality for creating, transforming, and processing streams, enabling efficient manipulation of sequential data.\n\n## Concept\n\nA `Stream` consists of a *Source* element that generates data, followed by zero or more *Pipe* elements that transform the data, and concludes with a *Sink* element that consumes the data to produce a result.\n\nThe *Source* is a lazy data structure that represents a sequence of values. It enables on-demand evaluation, making it highly efficient for processing large or infinite data sets. The library includes several built-in sources, such as **from_list**, **from_range**, and **from_tick**, which all follow the naming convention of starting with **from_**.\n\nThe *Pipe* elements are functions that transform the data in the stream. They can be used to map, or filter the data, among other operations like executing side effects. The library provides a variety of built-in pipe functions, such as **map**, **filter**, and **take**, which can be combined to create complex data processing pipelines.\n\nThe *Sink* is a function that processes data from a stream to generate a result. It can collect data into a list, subject or fold in a single value. Additionally, a Sink can terminate an infinite stream by consuming elements until a specific termination condition is met, such as **to_nil_error_terminated**.\n\n## Installation\n\nAdd `gs` to your Gleam project:\n\n```sh\ngleam add gs\n```\n\n## Usage\n\nCheck out the `/examples` directory for sample implementations.\nFurther documentation can be found at &lt;https://hexdocs.pm/gs&gt;.\n\n## License\n\nThis project is licensed under the Apache License, Version 2.0. See the [LICENSE](LICENSE) file for details.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"gs","title":"OverflowStrategy","content":"pub type OverflowStrategy {\n  Wait\n  Drop\n  Stop\n  Panic\n}\n\nWait\n\n\nDrop\n\n\nStop\n\n\nPanic\n\n\nSynonyms:\ngs.OverflowStrategy\ngs OverflowStrategy","url":"gs.html#OverflowStrategy"},{"doc":"gs","title":"Stream","content":"pub type Stream(a) {\n  Stream(pull: fn() -&gt; Option(#(a, Stream(a))))\n}\n A Stream represents a lazy, pull-based sequence of elements.\n Each element is computed only when requested, making it memory efficient\n for large or infinite sequences.\n\n Streams operate by &quot;pulling&quot; one element at a time:\n\n \n  [Source] --&gt; [Element 1] --&gt; [Element 2] --&gt; [Element 3] --&gt; None\n                  ^              ^              ^\n                  |              |              |\n             Pull when      Pull when      Pull when \n              needed         needed         needed\n \n\n Key characteristics:\n - Pull-based: Elements are computed on demand\n - Sequential: Only one element is processed at a time\n - Lazy: Elements are not computed until requested\n - Finite: Stream ends when None is returned\n\nStream(pull: fn() -&gt; Option(#(a, Stream(a))))\n\n\nSynonyms:\ngs.Stream\ngs Stream","url":"gs.html#Stream"},{"doc":"gs","title":"buffer","content":"pub fn buffer(\n  stream: Stream(a),\n  size: Int,\n  mode: OverflowStrategy,\n) -&gt; Stream(a)\n Creates a buffered stream that holds elements in a queue with specified overflow strategy.\n \n ## Example\n \n &gt; from_range(1, 10)\n |&gt; buffer(\n     size: 3,            // Buffer capacity\n     mode: Wait          // Wait when buffer is full\n )\n |&gt; tap(fn(x) { process.sleep(100) }) // Simulate slow processing\n |&gt; to_list()\n // Returns [1, 2, 3, 4, 5, 6, 7, 8, 9, 10] with buffering\n \n \n ## Visual Representation\n \n Fast producer, slow consumer with buffer:\n \n           +----------------Buffer (size 3)----------------+\n           |  +---+    +---+    +---+                    |\n Producer  |  | 1 |    | 2 |    | 3 |    (waiting...)   |  Consumer\n ---------&gt;|  +---+    +---+    +---+                    |----------&gt;\n [4,5,6,..]|  └─────────────────────┘                    | [1,2,3,..]\n           +--------------------------------------------+\n \n Strategy behaviors when buffer is full:\n - Wait:  Producer pauses until space available\n - Drop:  New elements are discarded\n - Stop:  Stream terminates\n - Panic: Raises an error\n \n \n ## When to Use\n - When processing speeds differ between producer and consumer\n - For implementing backpressure mechanisms\n - When you need to smooth out processing spikes\n - For rate-limiting or flow control\n - When implementing producer-consumer patterns\n - For managing memory usage with fast producers\n - When implementing stream processing pipelines\n \n ## Description\n The `buffer` function creates a buffered stream that can hold a specified\n number of elements in a queue. It&#39;s particularly useful when dealing with\n producers and consumers operating at different speeds. The function takes:\n 1. A stream to buffer\n 2. A buffer size limit\n 3. An overflow strategy specifying behavior when the buffer is full\n \n Overflow Strategies:\n - `Wait`: Producer waits until space is available (backpressure)\n - `Drop`: New elements are discarded when buffer is full\n - `Stop`: Stream terminates when buffer overflows\n - `Panic`: Raises an error on buffer overflow\n \n The buffering mechanism:\n - Creates an asynchronous queue of specified size\n - Processes elements through the queue in FIFO order\n - Applies the specified overflow strategy when full\n - Maintains element order\n - Handles stream termination gracefully\n \n This is particularly useful for:\n - Managing producer-consumer speed differences\n - Implementing backpressure\n - Controlling memory usage\n - Smoothing out processing irregularities\n - Building robust streaming pipelines\nSynonyms:\ngs.buffer\ngs buffer","url":"gs.html#buffer"},{"doc":"gs","title":"chunks","content":"pub fn chunks(stream: Stream(a), size: Int) -&gt; Stream(List(a))\n Groups elements from a stream into chunks of the specified size.\n \n ## Example\n \n &gt; from_range(1, 10)\n |&gt; chunks(3)\n |&gt; to_list()\n [[1, 2, 3], [4, 5, 6], [7, 8, 9], [10]]\n \n \n ## Visual Representation\n \n Input:  [1]--&gt;[2]--&gt;[3]--&gt;[4]--&gt;[5]--&gt;[6]--&gt;[7]--&gt;[8]--&gt;[9]--&gt;[10]--&gt;|\n          |     |     |     |     |     |     |     |     |\n          └─────┴─────┴     └─────┴─────┘     └─────┴─────┘     └─┘\n             chunk 1            chunk 2          chunk 3       chunk 4\n                ↓                 ↓                 ↓            ↓\n Output:    [[1,2,3]]--------&gt;[[4,5,6]]--------&gt;[[7,8,9]]-----&gt;[[10]]--&gt;|\n \n Where `|` represents the end of the stream and partial chunks are included\n \n ## When to Use\n - When processing data in fixed-size batches\n - For implementing pagination or windowing operations\n - When buffering or batching stream elements for bulk processing\n - For grouping elements for parallel processing\n - When implementing network protocols with fixed-size frames\n - For chunking large datasets into manageable pieces\n \n ## Description\n The `chunks` function transforms a stream by grouping its elements into \n fixed-size chunks. Each chunk is emitted as a list containing `size` elements,\n except possibly the last chunk which may contain fewer elements if the stream\n length is not evenly divisible by the chunk size.\n \n The function:\n - Processes elements lazily, only forming chunks when requested\n - Preserves the order of elements within and between chunks\n - Returns empty stream if size &lt;= 0\n - Includes partial chunks at the end of the stream\n - Never creates chunks larger than the specified size\n - Creates a stream of lists, where each list is a chunk\nSynonyms:\ngs.chunks\ngs chunks","url":"gs.html#chunks"},{"doc":"gs","title":"concat","content":"pub fn concat(first: Stream(a), second: Stream(a)) -&gt; Stream(a)\n Concatenates two streams into a single stream, yielding all elements from the first stream\n followed by all elements from the second stream.\n \n ## Example\n \n &gt; [1, 2, 3]\n |&gt; from_list\n |&gt; concat(from_list([4, 5, 6]))\n |&gt; to_list()\n [1, 2, 3, 4, 5, 6]\n \n \n ## Visual Representation\n \n Stream 1: [1]--&gt;[2]--&gt;[3]--&gt;|\n Stream 2: [4]--&gt;[5]--&gt;[6]--&gt;|\n \n           [1]--&gt;[2]--&gt;[3]--&gt;[4]--&gt;[5]--&gt;[6]--&gt;|\n           └── Stream 1 ──┘  └── Stream 2 ──┘\n \n Where `|` represents the end of the stream\n \n ## When to Use\n - When you need to combine two streams sequentially\n - For appending data from different sources\n - When implementing stream operations that need to chain multiple streams\n - When building composite streams from smaller streams\n - For implementing stream operations like `flat_map` or `flatten`\n - When you want to process multiple streams in sequence\n \n ## Description\n The `concat` function combines two streams by first yielding all elements from\n the first stream until it&#39;s exhausted, then yielding all elements from the\n second stream. The concatenation is lazy - elements from the second stream\n aren&#39;t processed until all elements from the first stream have been consumed.\n \n The function:\n - Preserves the order of elements from both streams\n - Processes streams lazily\n - Returns an empty stream if both input streams are empty\n - Handles infinite streams in the first position correctly (second stream never reached)\nSynonyms:\ngs.concat\ngs concat","url":"gs.html#concat"},{"doc":"gs","title":"count","content":"pub fn count(stream: Stream(a)) -&gt; Stream(#(a, Int))\n Counts elements as they pass through the stream, yielding tuples of elements and their count.\n \n ## Example\n \n &gt; from_range(1, 3)\n |&gt; count()\n |&gt; to_list()\n [#(1, 1), #(2, 2), #(3, 3)]\n \n \n ## Visual Representation\n \n Input:     [1]----&gt;[2]----&gt;[3]--&gt;|\n             |       |       |\n         count=1  count=2  count=3\n             |       |       |\n             v       v       v\n Output: [(1,1)]-&gt;[(2,2)]-&gt;[(3,3)]--&gt;|\n \n Where:\n - `|` represents the end of the stream\n - Each element is paired with its count\n - Count increases monotonically\n \n ## When to Use\n - When you need to track how many elements have been processed\n - For implementing progress tracking in stream processing\n - When you need element indices in a stream\n - For debugging or monitoring stream flow\n - When implementing stateful stream operations\n - For adding sequence numbers to stream elements\n \n ## Description\n The `count` function transforms a stream by pairing each element with a\n running count of how many elements have been processed. It maintains an\n internal counter that increments for each element, starting from 1.\n \n Key characteristics:\n - Preserves original elements\n - Adds monotonic counting\n - Processes elements lazily\n - Count starts at 1\n - Returns tuples of (element, count)\nSynonyms:\ngs.count\ngs count","url":"gs.html#count"},{"doc":"gs","title":"debug","content":"pub fn debug(stream: Stream(a)) -&gt; Stream(a)\n Logs each element of a stream for debugging.\n \n Example:\n \n pure(42) |&gt; debug\n \nSynonyms:\ngs.debug\ngs debug","url":"gs.html#debug"},{"doc":"gs","title":"drop","content":"pub fn drop(stream: Stream(a), n: Int) -&gt; Stream(a)\n Drops (skips) the first `n` elements from a stream.\n \n ## Example\n \n &gt; from_range(1, 10)\n |&gt; drop(5)\n |&gt; to_list()\n [6, 7, 8, 9, 10]\n \n \n ## Visual Representation\n \n Input:  [1]--&gt;[2]--&gt;[3]--&gt;[4]--&gt;[5]--&gt;[6]--&gt;[7]--&gt;[8]--&gt;|\n          ×     ×     ×     ×     ×     |     |     |\n                                        v     v     v\n Output: -----------------------------&gt;[6]--&gt;[7]--&gt;[8]--&gt;|\n \n Where:\n - `|` represents the end of the stream\n - `×` indicates dropped elements\n - Elements after the drop point flow through unchanged\n \n ## When to Use\n - When you want to skip a known number of elements at the start of a stream\n - For implementing pagination (skipping the first n pages)\n - When processing data streams where header elements should be ignored\n - For removing initialization or warm-up data\n - When implementing offset-based operations\n - To synchronize streams by dropping initial mismatched elements\n \n ## Description\n The `drop` function creates a new stream that skips the first `n` elements\n from the input stream and yields all remaining elements. The function processes\n elements lazily - only advancing through the dropped elements when the resulting\n stream is pulled. This makes it memory efficient as dropped elements are never\n materialized.\n \n The function:\n - Preserves the order of remaining elements\n - Processes dropped elements lazily\n - Returns an empty stream if n is greater than the stream length\n - Returns the original stream if n is 0 or negative\nSynonyms:\ngs.drop\ngs drop","url":"gs.html#drop"},{"doc":"gs","title":"error_terminated","content":"pub fn error_terminated(\n  stream: Stream(Result(a, b)),\n) -&gt; Stream(a)\n Terminates a stream when encountering an Error value, yielding only Ok values until then.\n \n ## Example\n \n &gt; [Ok(1), Ok(2), Error(&quot;oops&quot;), Ok(3)]\n |&gt; from_list\n |&gt; error_terminated\n |&gt; to_list()\n [1, 2]  // Terminates at Error, ignoring Ok(3)\n \n \n ## Visual Representation\n \n Input:   [Ok(1)]--&gt;[Ok(2)]--&gt;[Error(&quot;oops&quot;)]--&gt;[Ok(3)]--&gt;|\n             |         |            |              ×\n             v         v            v\n Output:    [1]------&gt;[2]----------&gt;|\n \n Where:\n - `|` represents the end of the stream\n - `×` represents values that are ignored after Error\n - Values flow from left to right until Error\n \n ## When to Use\n - When processing streams that should stop on first error\n - For implementing fail-fast error handling\n - When working with fallible operations where errors should terminate processing\n - For converting result-based sequences into regular streams\n - When implementing error-based early termination\n - For handling streams with error conditions as termination points\n \n ## Description\n The `error_terminated` function transforms a stream of Result values into a\n stream that yields only the unwrapped Ok values and terminates when it\n encounters an Error. Any values after the first Error are ignored. This is\n particularly useful when implementing fail-fast error handling or when working\n with streams that should terminate on first error condition.\n \n Key characteristics:\n - Unwraps Ok values into regular values\n - Terminates immediately on first Error\n - Ignores all values after Error\n - Processes elements lazily\n - Preserves order of elements before Error\n - Returns empty stream if first element is Error\nSynonyms:\ngs.error_terminated\ngs error_terminated","url":"gs.html#error_terminated"},{"doc":"gs","title":"filter","content":"pub fn filter(\n  stream: Stream(a),\n  keeping pred: fn(a) -&gt; Bool,\n) -&gt; Stream(a)\n Filters elements from a stream based on a predicate function.\n \n ## Example\n \n &gt; from_range(1, 10)\n |&gt; filter(fn(x) { x &gt; 5 })\n |&gt; to_list()\n [6, 7, 8, 9, 10]\n \n \n ## Visual Representation\n \n Input:  [1]--&gt;[2]--&gt;[3]--&gt;[4]--&gt;[5]--&gt;[6]--&gt;[7]--&gt;[8]--&gt;|\n          |     |     |     |     |     |     |     |\n        p(x)   p(x)  p(x)  p(x)  p(x)  p(x)  p(x)  p(x)\n          ×     ×     ×     ×     ×     ✓     ✓     ✓\n                                         |     |     |\n                                         v     v     v\n Output: ------------------------------&gt;[6]--&gt;[7]--&gt;[8]--&gt;|\n \n Where:\n - `|` represents the end of the stream\n - `p(x)` is the predicate function\n - `×` indicates the element was filtered out\n - `✓` indicates the element passed the filter\n \n ## When to Use\n - When you need to remove unwanted elements from a stream\n - For implementing search or query operations\n - When validating or sanitizing data streams\n - For extracting specific patterns or values\n - When implementing business logic filters\n - For data cleanup operations\n \n ## Description\n The `filter` function creates a new stream that only includes elements from\n the input stream that satisfy the given predicate function. Elements are\n processed lazily - the predicate is only evaluated when elements are pulled\n from the resulting stream.\n \n The function:\n - Preserves the order of elements that pass the filter\n - Skips elements that don&#39;t satisfy the predicate\n - Processes elements lazily (on-demand)\n - Returns an empty stream if no elements satisfy the predicate\nSynonyms:\ngs.filter\ngs filter","url":"gs.html#filter"},{"doc":"gs","title":"find","content":"pub fn find(stream: Stream(a), pred: fn(a) -&gt; Bool) -&gt; Stream(a)\n Finds the first element in a stream that satisfies a predicate and returns a stream containing only that element.\n \n ## Example\n \n &gt; from_range(1, 10)\n |&gt; find(fn(x) { x &gt; 5 })\n |&gt; to_list()\n [6]\n \n \n ## Visual Representation\n \n Input:  [1]--&gt;[2]--&gt;[3]--&gt;[4]--&gt;[5]--&gt;[6]--&gt;[7]--&gt;[8]--&gt;|\n          |     |     |     |     |     |     |     |\n        p(x)   p(x)  p(x)  p(x)  p(x)  p(x)  p(x)  p(x)\n          ×     ×     ×     ×     ×     ✓     -     -\n                                         |\n                                         v\n Output: ------------------------------&gt;[6]--&gt;|\n \n Where:\n - `|` represents the end of the stream\n - `p(x)` is the predicate function\n - `×` indicates the element did not match\n - `✓` indicates the first matching element\n - `-` indicates elements not evaluated\n \n ## When to Use\n - When searching for the first occurrence of an element meeting specific criteria\n - For implementing early termination in stream processing\n - When you need only the first matching element from a potentially large stream\n - For implementing search functionality\n - When validating the existence of elements meeting certain conditions\n - To optimize performance by stopping processing after finding a match\n \n ## Description\n The `find` function creates a new stream that contains only the first element\n from the input stream that satisfies the given predicate function. The function\n processes elements lazily and stops as soon as a matching element is found,\n making it efficient for large streams. If no element satisfies the predicate,\n an empty stream is returned.\n \n The function:\n - Processes elements sequentially until a match is found\n - Stops processing after finding the first match\n - Returns a stream with at most one element\n - Returns an empty stream if no elements match the predicate\nSynonyms:\ngs.find\ngs find","url":"gs.html#find"},{"doc":"gs","title":"flat_map","content":"pub fn flat_map(\n  over stream: Stream(a),\n  with f: fn(a) -&gt; Stream(b),\n) -&gt; Stream(b)\n Transforms each element into a stream and flattens all streams into a single stream.\n \n ## Example\n \n &gt; from_range(1, 3)\n |&gt; flat_map(fn(x) { from_range(1, x) })\n |&gt; to_list()\n [1, 1, 2, 1, 2, 3]\n \n \n ## Visual Representation\n \n Input:  [1]--------&gt;[2]--------&gt;[3]-------&gt;|\n          |           |           |\n          v           v           v\n         [1]--&gt;|    [1,2]--&gt;|   [1,2,3]--&gt;|\n          |       |   |     |   |   |   |\n          v       v   v     v   v   v   v\n Output: [1]----&gt;[1]--[2]--&gt;[1]--[2]--[3]--&gt;|\n \n Where `|` represents the end of the stream\n \n ## When to Use\n - When you need to generate multiple elements from each input element\n - For handling nested or hierarchical data structures\n - When implementing tree traversal or graph algorithms\n - For expanding or exploding single elements into sequences\n - When working with relationships where one item maps to many items\n - For implementing more complex stream transformations\n \n ## Description\n The `flat_map` function applies a transformation to each element of the input\n stream, where the transformation produces a new stream for each element. These\n resulting streams are then flattened into a single output stream. The function\n processes elements lazily, only generating new streams as elements are requested\n from the result stream.\n \n Key characteristics:\n - One-to-many transformation: Each input element can produce multiple output elements\n - Preserves order: Elements from earlier streams appear before elements from later streams\n - Lazy evaluation: Streams are only generated and flattened when needed\nSynonyms:\ngs.flat_map\ngs flat_map","url":"gs.html#flat_map"},{"doc":"gs","title":"flatten","content":"pub fn flatten(stream: Stream(Stream(a))) -&gt; Stream(a)\n Flattens a stream of streams into a single stream.\n \n ## Example\n \n &gt; [[1, 2], [3, 4], [5, 6]]\n |&gt; from_list\n |&gt; map(from_list)\n |&gt; flatten\n |&gt; to_list\n [1, 2, 3, 4, 5, 6]\n \n \n ## Visual Representation\n \n Input stream of streams:\n     +----------+    +----------+    +----------+\n  --&gt;|[1,2]     |---&gt;|[3,4]     |---&gt;|[5,6]     |--&gt;|\n     +----------+    +----------+    +----------+\n          |              |              |\n          v              v              v\n     [1]--&gt;[2]--&gt;| [3]--&gt;[4]--&gt;| [5]--&gt;[6]--&gt;|\n \n Flattened output:\n     +---+    +---+    +---+    +---+    +---+    +---+\n  --&gt;| 1 |---&gt;| 2 |---&gt;| 3 |---&gt;| 4 |---&gt;| 5 |---&gt;| 6 |--&gt;|\n     +---+    +---+    +---+    +---+    +---+    +---+\n \n Where `|` represents the end of the stream\n \n ## When to Use\n - When working with nested stream structures that need to be linearized\n - For processing hierarchical data as a single sequence\n - When implementing monadic operations (e.g., flat_map)\n - For combining multiple data sources into a single stream\n - When dealing with streams of collections that need to be concatenated\n - For implementing stream operations that produce multiple elements per input\n \n ## Description\n The `flatten` function takes a stream of streams and flattens it into a single\n stream by concatenating all inner streams in order. It processes streams lazily,\n only pulling from the next inner stream when all elements from the current inner\n stream have been consumed.\n \n The function:\n - Preserves the order of elements from both outer and inner streams\n - Processes streams lazily (on-demand)\n - Handles empty inner streams by skipping to the next one\n - Returns an empty stream if all inner streams are empty\n - Maintains the streaming nature of both levels\nSynonyms:\ngs.flatten\ngs flatten","url":"gs.html#flatten"},{"doc":"gs","title":"from_counter","content":"pub fn from_counter(start: Int) -&gt; Stream(Int)\n Given a counter integer, constructs an infinite stream of integers starting from\n the counter and incrementing by 1.\n\n ## Example\n\n \n &gt; let numbers = from_counter(1)\n &gt; numbers |&gt; take(3) |&gt; to_list()\n [1, 2, 3]\n \n\n ## Visual Representation\n\n \n from_counter(1)\n\n     +---+    +---+    +---+    +---+\n  --&gt;| 1 |---&gt;| 2 |---&gt;| 3 |---&gt;| 4 |--&gt;...\n     +---+    +---+    +---+    +---+\n \n\n ## Description\n\n Creates an infinite stream that yields consecutive integers starting from\n the provided counter value. Each element in the stream is exactly one greater\n than the previous element. The stream never terminates and will continue\n producing values indefinitely.\n\n ## When to Use\n\n - When you need a source of sequential integers\n - For generating unique identifiers\n - For creating test data with sequential values\n - When implementing pagination or indexing\n\n Note: Since this creates an infinite stream, make sure to use it with\n functions like `take`, `take_while`, or other stream terminators to avoid\n infinite loops.\nSynonyms:\ngs.from_counter\ngs from_counter","url":"gs.html#from_counter"},{"doc":"gs","title":"from_dict","content":"pub fn from_dict(dict: Dict(a, b)) -&gt; Stream(#(a, b))\n Creates a new stream from a dictionary.\n \n ## Example\n \n &gt; dict.from_list([#(&quot;a&quot;, 1), #(&quot;b&quot;, 2), #(&quot;c&quot;, 3)])\n |&gt; from_dict\n |&gt; to_list\n [#(&quot;a&quot;, 1), #(&quot;b&quot;, 2), #(&quot;c&quot;, 3)]\n \n \n ## Visual Representation\n \n dict.from_list([#(&quot;a&quot;, 1), #(&quot;b&quot;, 2), #(&quot;c&quot;, 3)]) |&gt; from_dict\n \n     +--------+    +--------+    +--------+\n  --&gt;|&quot;a&quot;: 1  |---&gt;|&quot;b&quot;: 2  |---&gt;|&quot;c&quot;: 3  |--&gt;|\n     +--------+    +--------+    +--------+\n \n Where `|` represents the end of the stream\n \n ## When to Use\n - When you need to process dictionary entries one at a time\n - When converting a dictionary to a stream of key-value pairs\n - When you want to lazily iterate over dictionary entries\n - When implementing operations that need to work with dictionaries as streams\n \n ## Description\n The `from_dict` function creates a stream that yields each key-value pair from \n the input dictionary as a tuple. The stream preserves the dictionary&#39;s entries\n but processes them lazily, one at a time. Each element in the resulting stream\n is a tuple containing a key and its associated value. The stream terminates\n after yielding all dictionary entries. This is particularly useful when you\n want to process large dictionaries efficiently or when you need to chain\n dictionary operations in a streaming context.\nSynonyms:\ngs.from_dict\ngs from_dict","url":"gs.html#from_dict"},{"doc":"gs","title":"from_empty","content":"pub fn from_empty() -&gt; Stream(a)\n Creates a new empty stream.\n \n The stream will yield nothing and terminate immediately.\n \n ## Example\n \n let empty_stream = from_empty()\n list.from_stream(empty_stream) // -&gt; []\n \n \n ## Visualization\n \n from_empty() -&gt; |\n \n Where `|` represents the end of the stream\n \n ## When to use\n - When you need a stream that yields no elements\n - As a base case for recursive stream operations\n - When implementing stream operations that might need to return an empty result\n - As a neutral element in stream concatenation operations\n \n ## Description\n The `from_empty` function creates a stream that immediately terminates without\n yielding any values. This is useful as a base case in recursive operations,\n when implementing stream combinators, or when you need to represent the absence\n of values in a streaming context. The empty stream is analogous to an empty\n list but in a lazy evaluation context.\nSynonyms:\ngs.from_empty\ngs from_empty","url":"gs.html#from_empty"},{"doc":"gs","title":"from_list","content":"pub fn from_list(items: List(a)) -&gt; Stream(a)\n Creates a new stream from a list.\n \n ## Example\n \n &gt; [1, 2, 3] |&gt; from_list |&gt; to_list\n [1, 2, 3]\n \n \n ## Visual Representation\n \n [1, 2, 3] |&gt; from_list\n \n     +---+    +---+    +---+\n  --&gt;| 1 |---&gt;| 2 |---&gt;| 3 |--&gt;|\n     +---+    +---+    +---+\n \n Where `|` represents the end of the stream\n \n ## When to Use\n - When converting an existing list to a stream\n - When you want to process list elements lazily\n - When implementing stream operations that build from lists\n - When you need to transform eager evaluation (lists) into lazy evaluation (streams)\n \n ## Description\n The `from_list` function creates a stream that yields each element from the input\n list in order. The stream maintains the original order of elements and terminates\n after yielding the last element. This is useful when you want to process list\n elements one at a time.\nSynonyms:\ngs.from_list\ngs from_list","url":"gs.html#from_list"},{"doc":"gs","title":"from_option","content":"pub fn from_option(option: Option(a)) -&gt; Stream(a)\n Creates a new stream from an option value.\n \n ## Example\n \n &gt; Some(42) |&gt; from_option |&gt; to_list\n [42]\n \n &gt; None |&gt; from_option |&gt; to_list\n []\n \n \n ## Visual Representation\n \n Some(42) |&gt; from_option\n \n     +----+\n  --&gt;| 42 |--&gt;|\n     +----+\n \n None |&gt; from_option\n \n  --&gt;|\n \n Where `|` represents the end of the stream\n \n ## When to Use\n - When converting an optional value into a stream\n - When you want to handle presence/absence of a value in a streaming context\n - As a building block for stream operations that may or may not have values\n \n ## Description\n The `from_option` function creates a stream from an Option value.\nSynonyms:\ngs.from_option\ngs from_option","url":"gs.html#from_option"},{"doc":"gs","title":"from_pure","content":"pub fn from_pure(value: a) -&gt; Stream(a)\n Creates a new single-element stream from a given value.\n \n ## Example\n \n &gt; 42 |&gt; from_pure |&gt; to_list\n [42]\n \n \n ## Visual Representation\n \n 42 |&gt; from_pure\n \n     +----+\n  --&gt;| 42 |--&gt;|\n     +----+\n \n Where `|` represents the end of the stream\n \n ## When to Use\n - When you need to create a stream with exactly one element\n - When converting a single value into a stream\n - As a building block for more complex stream operations\n - When implementing monadic operations that require lifting a value into a stream\n \n ## Description\n The `from_pure` function creates a stream that yields exactly one value and then\n terminates. It &quot;lifts&quot; a single value into the stream context. The resulting\n stream will emit the given value once and then immediately terminate. This is\n one of the fundamental stream constructors and is often used in combination\n with other stream operations.\nSynonyms:\ngs.from_pure\ngs from_pure","url":"gs.html#from_pure"},{"doc":"gs","title":"from_range","content":"pub fn from_range(from start: Int, to end: Int) -&gt; Stream(Int)\n Creates a stream of integers from a start value (inclusive) to an end value (inclusive).\n \n ## Example\n \n \n &gt; from_range(from: 1, to: 5) |&gt; to_list()\n [1, 2, 3, 4, 5]\n \n \n ## Visual Representation\n \n from_range(from: 1, to: 5)\n \n     +---+    +---+    +---+    +---+    +---+\n  --&gt;| 1 |---&gt;| 2 |---&gt;| 3 |---&gt;| 4 |---&gt;| 5 |--&gt;|\n     +---+    +---+    +---+    +---+    +---+\n \n Where `|` represents the end of the stream\n \n ## When to Use\n - When you need a finite sequence of consecutive integers\n - For iteration over a known range of numbers\n - For generating bounded sequences\n - When implementing pagination with fixed bounds\n - For creating test data with sequential values in a range\n \n ## Description\n The `from_range` function creates a finite stream that yields consecutive integers\n from the start value up to and including the end value. If the start value is\n greater than the end value, an empty stream is returned. Each element in the\n stream is exactly one greater than the previous element. The stream terminates\n after yielding the end value.\nSynonyms:\ngs.from_range\ngs from_range","url":"gs.html#from_range"},{"doc":"gs","title":"from_range_exclusive","content":"pub fn from_range_exclusive(\n  from start: Int,\n  until end: Int,\n) -&gt; Stream(Int)\n Creates a stream of integers from a start value (inclusive) to an end value (exclusive).\n \n ## Example\n \n \n &gt; from_range_exclusive(from: 1, until: 5) |&gt; to_list()\n [1, 2, 3, 4]\n \n \n ## Visual Representation\n \n from_range_exclusive(from: 1, until: 5)\n \n     +---+    +---+    +---+    +---+\n  --&gt;| 1 |---&gt;| 2 |---&gt;| 3 |---&gt;| 4 |--&gt;|\n     +---+    +---+    +---+    +---+\n \n Where `|` represents the end of the stream\n \n ## When to Use\n - When you need a finite sequence of consecutive integers excluding the end value\n - For zero-based indexing scenarios (e.g., array indices)\n - For loops where you want to exclude the upper bound\n - When implementing slice operations\n - For range operations that follow Python-style slicing conventions\n \n ## Description\n The `from_range_exclusive` function creates a finite stream that yields consecutive\n integers from the start value up to, but not including, the end value. If the\n start value is greater than or equal to the end value, an empty stream is returned.\n Each element in the stream is exactly one greater than the previous element.\n The stream terminates after yielding the last value before the end value.\nSynonyms:\ngs.from_range_exclusive\ngs from_range_exclusive","url":"gs.html#from_range_exclusive"},{"doc":"gs","title":"from_repeat","content":"pub fn from_repeat(value: a) -&gt; Stream(a)\n Creates an infinite stream that repeatedly yields the same value.\n\n ## Example\n \n &gt; 42 |&gt; from_repeat |&gt; take(3) |&gt; to_list\n [42, 42, 42]\n \n\n ## Visual Representation\n \n 42 |&gt; from_repeat\n\n     +----+    +----+    +----+    +----+\n  --&gt;| 42 |---&gt;| 42 |---&gt;| 42 |---&gt;| 42 |--&gt;...\n     +----+    +----+    +----+    +----+\n \n\n ## When to Use\n - When you need a constant stream of the same value\n - For testing stream operations with consistent input\n - When implementing backpressure mechanisms with default values\n - For creating placeholder or dummy data streams\n - When you need an infinite source of identical values\n\n ## Description\n The `from_repeat` function creates an infinite stream that yields the same\n value indefinitely. Each time the stream is pulled, it produces the original\n value and a new stream that will continue to produce the same value. This\n creates an endless sequence of identical values, useful for testing, default\n values, or as a building block for more complex stream operations. Since the\n stream is infinite, it should typically be used with functions like `take`\n or `take_while` to limit the number of values produced.\nSynonyms:\ngs.from_repeat\ngs from_repeat","url":"gs.html#from_repeat"},{"doc":"gs","title":"from_repeat_eval","content":"pub fn from_repeat_eval(f: fn() -&gt; a) -&gt; Stream(a)\n Creates an infinite stream that repeatedly evaluates a function to generate values.\n \n ## Example\n \n &gt; fn() { utils.timestamp() }\n |&gt; from_repeat_eval\n |&gt; take(3)\n |&gt; to_list\n // Emits [1705123456, 1705123457, 1705123458]  // Different timestamps\n \n \n ## Visual Representation\n \n from_repeat_eval(fn() { random() })\n \n     +-----+    +-----+    +-----+    +-----+\n  --&gt;| 42  |---&gt;| 17  |---&gt;| 33  |---&gt;| 89  |--&gt;...\n     +-----+    +-----+    +-----+    +-----+\n       ↑          ↑          ↑          ↑\n    f() call  f() call  f() call  f() call\n \n Where each value is generated by a fresh call to the function\n \n ## When to Use\n - When you need a stream of dynamically generated values\n - For creating streams of random numbers\n - For testing with varying data\n - When you need to encapsulate side-effects in a stream\n - For creating streams that reflect changing system state\n \n ## Description\n The `from_repeat_eval` function creates an infinite stream that generates values\n by repeatedly calling the provided function. Unlike `from_repeat` which produces\n the same value repeatedly, this function evaluates the given function each time\n a new value is needed, making it suitable for dynamic content generation.\n Each pull of the stream results in a fresh call to the function, potentially\n producing different values each time. The stream continues indefinitely and\n should typically be used with functions like `take` or `take_while` to limit\n the number of evaluations.\nSynonyms:\ngs.from_repeat_eval\ngs from_repeat_eval","url":"gs.html#from_repeat_eval"},{"doc":"gs","title":"from_result","content":"pub fn from_result(result: Result(a, b)) -&gt; Stream(a)\n Creates a new stream from a Result value.\n \n ## Example\n \n &gt; Ok(42) |&gt; from_result |&gt; to_list\n [42]\n \n &gt; Error(&quot;oops&quot;) |&gt; from_result |&gt; to_list\n []\n \n \n ## Visual Representation\n \n Ok(42) |&gt; from_result\n \n     +----+\n  --&gt;| 42 |--&gt;|\n     +----+\n \n Error(&quot;oops&quot;) |&gt; from_result\n \n  --&gt;|\n \n Where `|` represents the end of the stream\n \n ## When to Use\n - When converting a Result value into a stream\n - When handling computations that may fail in a streaming context\n - When filtering out error cases from a sequence of operations\n - When transforming error-handling code into data processing pipelines\n - As a building block for stream operations that work with Results\n \n ## Description\n The `from_result` function creates a stream from a Result value. If the Result\n is `Ok`, it creates a single-element stream containing the value. If the Result\n is `Error`, it creates an empty stream. This is useful for handling operations\n that may fail and converting them into a streaming context where errors are\n simply filtered out. The error value is discarded, making this function\n appropriate when you want to proceed with successful values only.\nSynonyms:\ngs.from_result\ngs from_result","url":"gs.html#from_result"},{"doc":"gs","title":"from_state_eval","content":"pub fn from_state_eval(\n  current: a,\n  f: fn(a) -&gt; #(b, a),\n) -&gt; Stream(b)\n Creates a stream by repeatedly applying a state transition function.\n \n ## Example\n \n // Generate Fibonacci numbers using state transitions\n from_state_eval(\n   #(0, 1),  // Initial state: (current, next)\n   fn(state) {\n     let #(current, next) = state\n     #(current, #(next, current + next))  // Return (value, new_state)\n   }\n )\n |&gt; take(8)\n |&gt; to_list()\n // Returns [0, 1, 1, 2, 3, 5, 8, 13]\n \n \n ## Visual Representation\n \n Initial State: (0,1)\n     +---+    +---+    +---+    +---+\n  --&gt;| 0 |---&gt;| 1 |---&gt;| 1 |---&gt;| 2 |--&gt;...\n     +---+    +---+    +---+    +---+\n       ↑        ↑        ↑        ↑\n    f(0,1)   f(1,1)   f(1,2)   f(2,3)\n       ↓        ↓        ↓        ↓\n    (1,1)    (1,2)    (2,3)    (3,5)\n    state    state    state    state\n \n Where:\n - Each element is produced by applying f to the current state\n - State transitions drive the stream generation\n - f returns both the current value and the next state\n \n ## When to Use\n - When generating sequences that depend on previous values\n - For implementing stateful stream transformations\n - When creating mathematical sequences (Fibonacci, etc.)\n - For simulating state machines or systems with evolving state\n - When implementing iterative algorithms that maintain state\n - For creating streams with complex progression rules\n - When you need to track auxiliary information between elements\n \n ## Description\n The `from_state_eval` function creates a stream by repeatedly applying a\n state transition function to an initial state. The function takes:\n 1. An initial state value of type `b`\n 2. A function `f` that takes the current state and returns a tuple of:\n    - The value to emit in the stream\n    - The next state to use\n \n Each time the stream is pulled, the state transition function is applied\n to the current state to produce both the next value and the next state.\n This allows for sophisticated sequence generation where each element can\n depend on the history of previous elements through the maintained state.\n \n The function is particularly useful for:\n - Generating mathematical sequences\n - Implementing stateful transformations\n - Creating streams with complex progression rules\n - Simulating systems with evolving state\n - Building iterative algorithms that require state\n \n The resulting stream continues indefinitely, driven by the state transitions,\n until terminated by other stream operations.\nSynonyms:\ngs.from_state_eval\ngs from_state_eval","url":"gs.html#from_state_eval"},{"doc":"gs","title":"from_subject","content":"pub fn from_subject(subject: Subject(Option(a))) -&gt; Stream(a)\n Creates a stream from a process Subject that receives values of type Option(a).\n \n ## Example\n \n let subject = process.new_subject()\n process.send(subject, Some(1))\n process.send(subject, Some(2))\n process.send(subject, None)\n \n subject\n |&gt; from_subject\n |&gt; to_list\n // -&gt; [1, 2]\n \n \n ## Visual Representation\n \n Subject --&gt; Stream\n \n      +---+    +---+    \n ----&gt;| 1 |---&gt;| 2 |---&gt; None\n      +---+    +---+    \n        ↑        ↑         ↑\n    Some(1)  Some(2)    None\n \n Where messages flow from the Subject into the Stream until None is received\n \n ## When to Use\n - When converting process messages into a stream\n - For handling asynchronous data sources\n - When bridging between process-based and stream-based code\n - For implementing pub/sub patterns with streams\n - When you need to process messages sequentially\n \n ## Description\n The `from_subject` function creates a stream that yields values received from\n a process Subject. It expects messages of type `Option(a)` where:\n - `Some(value)` represents a value to be emitted by the stream\n - `None` signals the end of the stream\n \n The stream will continuously wait for messages until a `None` is received,\n at which point it terminates. This makes it useful for converting\n asynchronous message-based communication into a sequential stream\n of values.\nSynonyms:\ngs.from_subject\ngs from_subject","url":"gs.html#from_subject"},{"doc":"gs","title":"from_subject_timeout","content":"pub fn from_subject_timeout(\n  subject: Subject(a),\n  timeout_ms: Int,\n) -&gt; Stream(a)\n Creates a stream from a process Subject that times out after a specified duration.\n \n ## Example\n \n let subject = process.new_subject()\n \n // In another process\n process.send(subject, 1)\n process.send(subject, 2)\n // Wait more than timeout...\n process.send(subject, 3)\n \n subject\n |&gt; from_subject_timeout(1000)\n |&gt; to_list\n // -&gt; [1, 2]  // 3 is not received due to timeout\n \n \n ## Visual Representation\n \n Subject --&gt; Stream (with timeout)\n \n      +---+    +---+         +---+\n ----&gt;| 1 |---&gt;| 2 |----X----| 3 |\n      +---+    +---+    |    +---+\n        ↑        ↑      |      ↑\n     0.1s     0.2s   1.0s    1.1s\n                      timeout\n \n Where &#39;X&#39; represents the timeout point that terminates the stream\n \n ## When to Use\n - When processing messages with time constraints\n - For implementing timeout-based protocols\n - When handling potentially slow or unreliable message sources\n - For graceful shutdown of stream processing\n - When implementing time-bounded operations\n - For preventing infinite waiting on message streams\n \n ## Description\n The `from_subject_timeout` function creates a stream that yields values received\n from a process Subject, but will terminate if no message is received within the\n specified timeout period (`timeout_ms`). Each message receipt resets the timeout\n window. This is particularly useful for handling scenarios where message flow\n might be interrupted or when you need to ensure timely processing of messages.\n \n The stream will:\n - Emit each received message as a stream element\n - Wait up to `timeout_ms` milliseconds for each new message\n - Terminate if no message is received within the timeout period\n - Reset the timeout window after each successful message receipt\nSynonyms:\ngs.from_subject_timeout\ngs from_subject_timeout","url":"gs.html#from_subject_timeout"},{"doc":"gs","title":"from_tick","content":"pub fn from_tick(delay_ms: Int) -&gt; Stream(Int)\n Creates a stream that emits a value every `delay_ms` milliseconds.\n \n ## Example\n \n \n &gt; 1000 \n |&gt; from_tick\n |&gt; take(3)  \n |&gt; to_list\n // Emits [0, 0, 0] with 1 second delay between each value\n \n \n ## Visual Representation\n \n from_tick(1000)\n \n     +---+    +---+    +---+    +---+\n  --&gt;| 0 |-1s&gt;| 0 |-1s&gt;| 0 |-1s&gt;| 0 |--&gt;...\n     +---+    +---+    +---+    +---+\n \n Where `-1s&gt;` represents a 1 second delay\n \n ## When to Use\n - When implementing polling mechanisms\n - For creating periodic events or heartbeats\n - When building timer-based functionality\n - For implementing rate-limiting or throttling\n - When simulating time-based events in testing\n \n ## Description\n The `from_tick` function creates an infinite stream that emits values at \n regular intervals specified by `delay_ms`. Each emission is accompanied by\n a delay of `delay_ms` milliseconds. The stream emits the number of\n milliseconds that were actually delayed beyond the expected interval\n (usually 0 unless system load causes delays).\n \n The function will panic if `delay_ms` is less than or equal to 0.\nSynonyms:\ngs.from_tick\ngs from_tick","url":"gs.html#from_tick"},{"doc":"gs","title":"from_timestamp_eval","content":"pub fn from_timestamp_eval() -&gt; Stream(Int)\n Creates an infinite stream of Unix timestamps that evaluates the current time on each pull.\n \n ## Example\n \n &gt; from_timestamp_eval()\n |&gt; take(3)\n |&gt; to_list\n // Emits [1705123456, 1705123457, 1705123458]  // Different timestamps\n \n \n ## Visual Representation\n \n from_timestamp_eval()\n \n     +---------+    +---------+    +---------+    +---------+\n  --&gt;| 1705123 |---&gt;| 1705124 |---&gt;| 1705125 |---&gt;| 1705126 |--&gt;...\n     +---------+    +---------+    +---------+    +---------+\n          ↑              ↑              ↑              ↑\n     timestamp()    timestamp()    timestamp()    timestamp()\n \n Where each value is a fresh timestamp evaluation\n \n ## When to Use\n - When you need a stream of current timestamps\n - For logging with temporal information\n - When implementing time-based monitoring\n - For creating time series data\n - When measuring elapsed time between operations\n \n ## Description\n The `from_timestamp_eval` function creates an infinite stream that generates \n Unix timestamps. Unlike a static stream, this evaluates the current timestamp\n each time a value is pulled, ensuring each emitted value reflects the actual\n time at the moment of access. The stream continues indefinitely and should\n typically be used with functions like `take` or `take_while` to limit the\n number of evaluations.\nSynonyms:\ngs.from_timestamp_eval\ngs from_timestamp_eval","url":"gs.html#from_timestamp_eval"},{"doc":"gs","title":"intersperse","content":"pub fn intersperse(stream: Stream(a), separator: a) -&gt; Stream(a)\n Inserts a separator element between each element of a stream.\n \n ## Example\n \n &gt; from_list([1, 2, 3])\n |&gt; intersperse(0)\n |&gt; to_list()\n [1, 0, 2, 0, 3]\n \n \n ## Visual Representation\n \n Input:  [1]-----&gt;[2]-----&gt;[3]--&gt;|\n \n Output: [1]--&gt;[0]---&gt;[2]---&gt;[0]---&gt;[3]--&gt;|\n          ^     ^      ^      ^      ^\n          |     |      |      |      |\n       value   sep   value   sep   value\n \n Where:\n - `|` represents the end of the stream\n - `sep` represents the separator\n - Elements flow from left to right\n \n ## When to Use\n - When formatting streams for display (e.g., adding commas between items)\n - When implementing string join operations with streams\n - For adding delimiters between stream elements\n - When creating visual separations in streamed output\n - For implementing protocol-specific element separation\n - When generating formatted text from stream elements\n \n ## Description\n The `intersperse` function creates a new stream that contains all elements\n from the input stream with a separator value inserted between each pair of\n elements. The separator is not added before the first element or after the\n last element. This is particularly useful when formatting streams for\n display or when implementing protocols that require specific delimiters\n between elements.\n \n Key characteristics:\n - Preserves original element order\n - Only inserts separators between elements\n - No separator before first or after last element\n - Processes elements lazily\n - Returns empty stream if input is empty\nSynonyms:\ngs.intersperse\ngs intersperse","url":"gs.html#intersperse"},{"doc":"gs","title":"map","content":"pub fn map(\n  over stream: Stream(a),\n  with f: fn(a) -&gt; b,\n) -&gt; Stream(b)\n Maps a function over a stream.\n \n ## Example\n \n &gt; from_range(1, 5)\n |&gt; map(fn(x) { x * 2 })\n |&gt; to_list()\n [2, 4, 6, 8, 10]\n \n \n ## Visual Representation\n \n stream:    [1]--&gt;[2]--&gt;[3]--&gt;[4]--&gt;[5]--&gt;|\n             |     |     |     |     |\n           f(x)   f(x)  f(x)  f(x)  f(x)\n             |     |     |     |     |\n             v     v     v     v     v\n result:    [2]--&gt;[4]--&gt;[6]--&gt;[8]--&gt;[10]--&gt;|\n \n Where `|` represents the end of the stream and `f(x)` is the mapping function\n \n ## When to Use\n - When transforming each element of a stream without changing the stream structure\n - For data conversion or formatting\n - When applying calculations to each element\n - For type conversion operations\n - When preparing data for further processing\n - As a building block in larger stream processing pipelines\n \n ## Description\n The `map` function transforms a stream by applying a function to each element,\n creating a new stream with the transformed values. The stream maintains its\n original structure (length and order) while transforming the elements. The\n mapping function is applied lazily - only when elements are pulled from the\n resulting stream.\nSynonyms:\ngs.map\ngs map","url":"gs.html#map"},{"doc":"gs","title":"none_terminated","content":"pub fn none_terminated(stream: Stream(Option(a))) -&gt; Stream(a)\n Creates a stream that terminates when encountering a None value.\n \n ## Example\n \n &gt; [Some(1), Some(2), None, Some(3)]\n |&gt; from_list\n |&gt; none_terminated\n |&gt; to_list()\n [1, 2]  // Terminates at None, ignoring Some(3)\n \n \n ## Visual Representation\n \n Input:   [Some(1)]--&gt;[Some(2)]--&gt;[None]--&gt;[Some(3)]--&gt;|\n             |           |           |         ×\n             v           v           v\n Output:    [1]--------&gt;[2]--------&gt;|\n \n Where:\n - `|` represents the end of the stream\n - `×` represents values that are ignored after None\n - Values flow from left to right until None\n \n ## When to Use\n - When processing streams that use None as a termination signal\n - For implementing protocols with end-of-stream markers\n - When working with optional values where None indicates completion\n - For converting option-based sequences into regular streams\n - When implementing early termination based on None values\n - For handling streams with natural termination points\n \n ## Description\n The `none_terminated` function transforms a stream of Option values into a\n stream that yields only the unwrapped Some values and terminates when it\n encounters a None. Any values after the first None are ignored. This is\n particularly useful when working with protocols or data sources that use\n None as an end-of-stream marker.\n \n Key characteristics:\n - Unwraps Some values into regular values\n - Terminates immediately on first None\n - Ignores all values after None\n - Processes elements lazily\n - Preserves order of elements before None\n - Returns empty stream if first element is None\nSynonyms:\ngs.none_terminated\ngs none_terminated","url":"gs.html#none_terminated"},{"doc":"gs","title":"println","content":"pub fn println(stream: Stream(String)) -&gt; Stream(String)\n Prints each element of a stream to the console.\n \n Example:\n \n pure(&quot;Hello, world!&quot;) |&gt; println\n \nSynonyms:\ngs.println\ngs println","url":"gs.html#println"},{"doc":"gs","title":"rate_limit_backoff","content":"pub fn rate_limit_backoff(\n  stream: Stream(a),\n  initial_delay_ms: Int,\n  max_delay_ms: Int,\n  factor: Float,\n) -&gt; Stream(a)\n Implements exponential backoff rate limiting for a stream by dynamically adjusting\n delays between elements based on processing history.\n \n ## Example\n \n &gt; from_range(1, 10)\n |&gt; rate_limit_backoff(\n     initial_delay_ms: 100,  // Start with 100ms delay\n     max_delay_ms: 1000,    // Cap at 1 second delay\n     factor: 2.0           // Double delay on each retry\n )\n |&gt; to_list()\n // Returns [1, 2, 3, 4, 5, 6, 7, 8, 9, 10] with increasing delays\n \n \n ## Visual Representation\n \n Input:  [1]--&gt;[2]--&gt;[3]--&gt;[4]--&gt;[5]--&gt;[6]--&gt;...\n          |     |     |     |     |     |\n       100ms  200ms 400ms 800ms  1s    1s    (exponential delays)\n          |     |     |     |     |     |\n Output: [1]--&gt;[2]--&gt;[3]--&gt;[4]--&gt;[5]--&gt;[6]--&gt;...\n \n \n ## When to Use\n - When implementing retry logic with increasing delays\n - For handling rate-limited APIs with automatic backoff\n - When implementing fault-tolerant stream processing\n - For graceful degradation under load\n - When implementing adaptive rate limiting\n - For implementing congestion control mechanisms\n \n ## Description\n The `rate_limit_backoff` function creates a rate-limited stream that automatically\n adjusts delays between elements using exponential backoff. Unlike linear rate\n limiting, this approach increases delays exponentially up to a maximum value,\n making it suitable for scenarios where adaptive rate limiting is needed.\n \n Parameters:\n - initial_delay_ms: Starting delay between elements\n - max_delay_ms: Maximum delay cap\n - factor: Multiplication factor for delay increase\n \n The function ensures that:\n - Delays start at initial_delay_ms\n - Each delay is multiplied by factor\n - Delays are capped at max_delay_ms\n \n This is particularly useful for:\n - API rate limit compliance\n - Retry mechanisms\n - Load balancing\n - Error recovery\n - System protection\nSynonyms:\ngs.rate_limit_backoff\ngs rate_limit_backoff","url":"gs.html#rate_limit_backoff"},{"doc":"gs","title":"rate_limit_linear","content":"pub fn rate_limit_linear(\n  stream: Stream(a),\n  rate: Int,\n  interval_ms: Int,\n) -&gt; Stream(a)\n TODO: rate_limit\n Implements linear rate limiting for a stream by controlling the number of elements processed per time interval.\n \n ## Example\n \n &gt; from_range(1, 10)\n |&gt; rate_limit_linear(\n     rate: 2,        // 2 elements\n     interval_ms: 1000  // per second\n )\n |&gt; to_list()\n // Returns [1, 2, 3, 4, 5, 6, 7, 8, 9, 10] with constant delays\n // Processes 2 elements per second\n \n \n ## Visual Representation\n \n Input:  [1]--&gt;[2]--&gt;[3]--&gt;[4]--&gt;[5]--&gt;[6]--&gt;...\n          |     |     |     |     |     |\n        500ms 500ms 500ms 500ms 500ms 500ms  (constant delays)\n          |     |     |     |     |     |\n Output: [1]--&gt;[2]--&gt;[3]--&gt;[4]--&gt;[5]--&gt;[6]--&gt;...\n         |&lt;-1 s.-&gt; | &lt;- 1 s. -&gt; |\n          (2 items)  (2 items)\n \n \n ## When to Use\n - When implementing consistent rate limiting for API calls\n - For controlling throughput to external services\n - When processing needs to match a specific rate requirement\n - For implementing SLA compliance\n - When preventing system overload\n - For simulating time-constrained processing\n - When implementing quota-based systems\n \n ## Description\n The `rate_limit_linear` function creates a rate-limited stream that processes\n a specified number of elements within a given time interval. Unlike exponential\n backoff, this maintains a constant rate of processing, making it suitable for\n scenarios where consistent throughput is required.\n \n Parameters:\n - rate: Number of elements to process per interval\n - interval_ms: Time interval in milliseconds\n \n The function ensures that:\n - Elements are processed at a constant rate\n - Delays are evenly distributed across the interval\n - Processing matches the specified throughput\n - Rate limiting is precise and predictable\n \n This is particularly useful for:\n - API rate limiting compliance\n - Resource utilization control\n - Service level agreement implementation\n - System load management\n - Throughput optimization\n - Quota enforcement\nSynonyms:\ngs.rate_limit_linear\ngs rate_limit_linear","url":"gs.html#rate_limit_linear"},{"doc":"gs","title":"recover","content":"pub fn recover(\n  stream: Stream(Result(a, b)),\n  recover: fn(b) -&gt; Stream(a),\n) -&gt; Stream(a)\n Recovers from an error in a stream using the given function.\n Alias for `try_recover`.\nSynonyms:\ngs.recover\ngs recover","url":"gs.html#recover"},{"doc":"gs","title":"sleep","content":"pub fn sleep(stream: Stream(a), delay_ms: Int) -&gt; Stream(a)\n Adds a delay between each element in a stream.\n \n ## Example\n \n &gt; from_range(1, 3)\n |&gt; sleep(1000)  // 1 second delay\n |&gt; to_list()\n // Returns [1, 2, 3] with 1 second delay between each element\n \n \n ## Visual Representation\n \n Input:  [1]--&gt;[2]--&gt;[3]--&gt;|\n \n Output: [1]--1s--&gt;[2]--1s--&gt;[3]--&gt;|\n          |         |         |\n          ↓         ↓         ↓\n        delay     delay     delay\n \n Where:\n - `|` represents the end of the stream\n - `--1s--&gt;` represents a 1 second delay\n - Elements flow from left to right with delays\n \n ## When to Use\n - When implementing rate limiting\n - For throttling high-speed streams\n - When simulating time-dependent processes\n - For creating animated or time-based sequences\n - When implementing backpressure mechanisms\n - For testing timing-dependent operations\n - When synchronizing with external time-based systems\n \n ## Description\n The `sleep` function creates a new stream that introduces a fixed delay\n between each element of the input stream. Every time an element is pulled\n from the stream, the function waits for the specified number of milliseconds\n before yielding the next element. This is useful for controlling the rate\n of stream processing or simulating time-dependent sequences.\n \n Key characteristics:\n - Maintains original element order\n - Adds consistent delays between elements\n - Processes elements lazily\n - Delay occurs before each element\n - Suitable for infinite streams\nSynonyms:\ngs.sleep\ngs sleep","url":"gs.html#sleep"},{"doc":"gs","title":"take","content":"pub fn take(stream: Stream(a), n: Int) -&gt; Stream(a)\n Takes the first `n` elements from a stream.\n \n ## Example\n \n &gt; from_range(1, 10)\n |&gt; take(5)\n |&gt; to_list()\n [1, 2, 3, 4, 5]\n \n \n ## Visual Representation\n \n Input:  [1]--&gt;[2]--&gt;[3]--&gt;[4]--&gt;[5]--&gt;[6]--&gt;[7]--&gt;[8]--&gt;|\n          |     |     |     |     |     ×     ×     ×\n          v     v     v     v     v\n Output: [1]--&gt;[2]--&gt;[3]--&gt;[4]--&gt;[5]--&gt;|\n \n Where:\n - `|` represents the end of the stream\n - `×` indicates elements that are not taken\n - Elements before the take limit flow through unchanged\n \n ## When to Use\n - When you need to limit the number of elements from a potentially infinite stream\n - For implementing pagination (taking the first n items)\n - When sampling or previewing data from a large stream\n - For testing with a subset of stream elements\n - When implementing batched processing\n - To prevent infinite processing of endless streams\n \n ## Description\n The `take` function creates a new stream containing only the first `n` elements\n from the input stream. Once `n` elements have been yielded, the resulting stream\n terminates, even if more elements are available in the input stream. This is\n particularly useful when working with infinite streams or when you need to limit\n the number of elements processed.\n \n The function:\n - Preserves the order of elements\n - Processes elements lazily\n - Returns less than n elements if the input stream is shorter\n - Returns an empty stream if n is 0 or negative\nSynonyms:\ngs.take\ngs take","url":"gs.html#take"},{"doc":"gs","title":"take_while","content":"pub fn take_while(\n  stream: Stream(a),\n  pred: fn(a) -&gt; Bool,\n) -&gt; Stream(a)\n Takes elements from a stream as long as they satisfy a predicate.\n \n ## Example\n \n &gt; from_range(1, 10)\n |&gt; take_while(fn(x) { x &lt;= 5 })\n |&gt; to_list()\n [1, 2, 3, 4, 5]\n \n \n ## Visual Representation\n \n Input:  [1]--&gt;[2]--&gt;[3]--&gt;[4]--&gt;[5]--&gt;[6]--&gt;[7]--&gt;[8]--&gt;|\n          |     |     |     |     |     ×     ×     ×\n        p(x)   p(x)  p(x)  p(x)  p(x)  p(x)\n          ✓     ✓     ✓     ✓     ✓     ×\n          v     v     v     v     v\n Output: [1]--&gt;[2]--&gt;[3]--&gt;[4]--&gt;[5]--&gt;|\n \n Where:\n - `|` represents the end of the stream\n - `p(x)` is the predicate function\n - `✓` indicates the predicate returned True\n - `×` indicates where the stream terminates (predicate returned False)\n \n ## When to Use\n - When you need to collect elements until a condition is no longer met\n - For processing sequences until a terminating condition occurs\n - When implementing range filters with dynamic end conditions\n - For handling sorted data where you want elements until a threshold\n - When processing time-series data until a specific event\n - For implementing early termination based on element values\n \n ## Description\n The `take_while` function creates a new stream that yields elements from the\n input stream as long as they satisfy the given predicate function. Once an\n element is encountered that fails the predicate, the stream terminates\n immediately, ignoring any remaining elements. This is particularly useful\n when working with sorted data or when you need to process elements until\n a certain condition is met.\n \n The function:\n - Processes elements lazily\n - Preserves the order of elements\n - Stops immediately when the predicate returns False\n - Returns an empty stream if the first element fails the predicate\n - Never processes elements after the first failing predicate\nSynonyms:\ngs.take_while\ngs take_while","url":"gs.html#take_while"},{"doc":"gs","title":"tap","content":"pub fn tap(stream: Stream(a), f: fn(a) -&gt; b) -&gt; Stream(a)\n Applies a function to each element of a stream for side effects.\n \n Example:\n \n repeat(1) |&gt; tap(fn(x) { io.println(x) })\n \nSynonyms:\ngs.tap\ngs tap","url":"gs.html#tap"},{"doc":"gs","title":"to_fold","content":"pub fn to_fold(\n  stream: Stream(a),\n  initial: b,\n  f: fn(b, a) -&gt; b,\n) -&gt; b\n Reduces a stream to a single value by applying a function to each element.\n \n ## Example\n \n &gt; from_range(1, 5)\n |&gt; to_fold(0, fn(acc, x) { acc + x })\n // Returns 15 (sum of 1 to 5)\n \n \n ## Visual Representation\n \n Input:     [1]----&gt;[2]----&gt;[3]----&gt;[4]----&gt;[5]--&gt;|\n             |       |       |       |       |\n           f(0,1)  f(1,2)  f(3,3)  f(6,4) f(10,5)\n             |       |       |       |       |\n             v       v       v       v       v\n Acc:     0 -&gt; 1 -&gt; 3 -&gt; 6 -&gt; 10 -&gt; 15\n \n Where:\n - `|` represents the end of the stream\n - `f(acc,x)` shows each fold operation\n - Values flow from left to right, accumulating results\n \n ## When to Use\n - When you need to combine all stream elements into a single value\n - For calculating aggregates (sum, product, etc.)\n - When implementing custom collection operations\n - For transforming streams into non-stream data structures\n - When you need to maintain state while processing a stream\n - For implementing reduction operations over streams\n \n ## Description\n The `to_fold` function processes a stream by applying a folding function to\n each element, maintaining an accumulator value throughout the operation. It:\n - Takes an initial accumulator value\n - Processes each stream element in order\n - Updates the accumulator using the provided function\n - Returns the final accumulator value\n \n The folding function receives two arguments:\n 1. The current accumulator value\n 2. The current stream element\n \n This is a terminal operation that consumes the entire stream and produces\n a single result. It&#39;s commonly used as the foundation for other stream\n termination operations like `to_list`, `to_set`, etc.\nSynonyms:\ngs.to_fold\ngs to_fold","url":"gs.html#to_fold"},{"doc":"gs","title":"to_list","content":"pub fn to_list(stream: Stream(a)) -&gt; List(a)\n Collects all elements from a stream into a list.\n \n ## Example\n \n &gt; from_range(1, 5)\n |&gt; to_list()\n [1, 2, 3, 4, 5]\n \n \n ## Visual Representation\n \n Input Stream:\n \n     +---+    +---+    +---+    +---+    +---+\n  --&gt;| 1 |---&gt;| 2 |---&gt;| 3 |---&gt;| 4 |---&gt;| 5 |--&gt;|\n     +---+    +---+    +---+    +---+    +---+\n       |       |        |        |        |\n       v       v        v        v        v\n      [1] -&gt; [1,2] -&gt; [1,2,3] -&gt; [1,2,3,4] -&gt; [1,2,3,4,5]\n \n Where:\n - `|` represents the end of the stream\n - Each element is accumulated into the growing list\n \n ## When to Use\n - When you need to collect all stream elements into memory\n - For processing finite streams where total size is manageable\n - When you need random access to stream elements\n - For converting lazy evaluation (streams) to eager evaluation (lists)\n - When implementing terminal operations that need all elements at once\n - For testing or debugging stream operations\n \n ## Description\n The `to_list` function is a terminal operation that consumes the entire stream\n and collects all elements into a list. It processes elements sequentially,\n maintaining their original order. This operation requires enough memory to\n hold all stream elements simultaneously, so it should be used carefully with\n large or infinite streams.\n \n Key characteristics:\n - Preserves element order\n - Consumes the entire stream\n - Requires memory proportional to stream length\n - Creates a new list containing all elements\n - Terminal operation (ends stream processing)\n - Suitable for finite streams\nSynonyms:\ngs.to_list\ngs to_list","url":"gs.html#to_list"},{"doc":"gs","title":"to_nil","content":"pub fn to_nil(stream: Stream(a)) -&gt; Nil\n Processes a stream completely, discarding all values and returning Nil.\n \n ## Example\n \n &gt; from_range(1, 5)\n |&gt; tap(io.println)\n |&gt; to_nil\n // Prints: 1, 2, 3, 4, 5\n // Returns: Nil\n \n \n ## Visual Representation\n \n Input:     [1]--&gt;[2]--&gt;[3]--&gt;[4]--&gt;[5]--&gt;|\n             ↓     ↓     ↓     ↓     ↓\n           process process process process process\n             ↓     ↓     ↓     ↓     ↓\n Output:    Nil   Nil   Nil   Nil   Nil\n \n Where:\n - `|` represents the end of the stream\n - Each element is processed but discarded\n - Only side effects (if any) remain\n \n ## When to Use\n - When you only care about side effects (logging, printing, etc.)\n - For consuming a stream without keeping results\n - When implementing fire-and-forget operations\n - For triggering stream processing without collecting results\n - When memory usage is critical and results aren&#39;t needed\n - For testing stream processing without result validation\n \n ## Description\n The `to_nil` function is a terminal operation that processes an entire stream\n but discards all values, returning Nil. It&#39;s particularly useful when you&#39;re\n interested in the side effects of processing a stream (like printing or\n logging) but don&#39;t need the actual values. This operation is memory efficient\n as it doesn&#39;t accumulate any results.\n \n Key characteristics:\n - Processes all elements sequentially\n - Discards all values\n - Returns Nil regardless of input\n - Preserves side effects from stream processing\n - Terminal operation (ends stream processing)\n - Memory efficient (no accumulation)\nSynonyms:\ngs.to_nil\ngs to_nil","url":"gs.html#to_nil"},{"doc":"gs","title":"to_nil_error_terminated","content":"pub fn to_nil_error_terminated(\n  stream: Stream(Result(a, b)),\n) -&gt; Nil\n Processes a stream of Results until encountering an Error, discarding all values and returning Nil.\n \n ## Example\n \n &gt; [Ok(1), Ok(2), Error(&quot;oops&quot;), Ok(3)]\n |&gt; from_list\n |&gt; to_nil_error_terminated\n // Processes 1, 2, stops at Error(&quot;oops&quot;), ignores 3\n // Returns: Nil\n \n \n ## Visual Representation\n \n Input:   [Ok(1)]--&gt;[Ok(2)]--&gt;[Error]--&gt;[Ok(3)]--&gt;|\n             |         |          |         ×\n             ↓         ↓          ↓\n            Nil       Nil        Nil\n                                  |\n Output:                         Nil\n \n Where:\n - `|` represents the end of the stream\n - `×` represents values that are ignored after Error\n - Each Ok value is processed but discarded\n - Processing stops at Error\n \n ## When to Use\n - When processing Result streams for side effects until first error\n - For consuming streams with Error as termination signal\n - When implementing fail-fast operations with side effects\n - For processing streams where errors indicate critical failures\n - When you need to execute side effects but don&#39;t need values\n - For testing error handling without result validation\n - When implementing fault-tolerant processing pipelines\n \n ## Description\n The `to_nil_error_terminated` function is a terminal operation that processes\n a stream of Results until it encounters an Error value. It combines the behavior\n of `error_terminated` and `to_nil`, processing each Ok value but discarding\n the results. The operation stops immediately when an Error is encountered,\n ignoring any subsequent elements.\n \n Key characteristics:\n - Processes Ok values sequentially until Error\n - Discards all values\n - Stops processing at first Error\n - Ignores elements after Error\n - Returns Nil regardless of input\n - Memory efficient (no accumulation)\n - Preserves side effects from stream processing\n - Terminal operation (ends stream processing)\nSynonyms:\ngs.to_nil_error_terminated\ngs to_nil_error_terminated","url":"gs.html#to_nil_error_terminated"},{"doc":"gs","title":"to_nil_none_terminated","content":"pub fn to_nil_none_terminated(stream: Stream(Option(a))) -&gt; Nil\n Processes a stream of Options until encountering None, discarding all values and returning Nil.\n \n ## Example\n \n [Some(1), Some(2), None, Some(3)]\n |&gt; from_list\n |&gt; to_nil_none_terminated\n // Processes 1, 2, stops at None, ignores 3\n // Returns: Nil\n \n \n ## Visual Representation\n \n Input:   [Some(1)]--&gt;[Some(2)]--&gt;[None]--&gt;[Some(3)]--&gt;|\n             |           |           |         ×\n             ↓           ↓           ↓\n            Nil        Nil         Nil\n                                    |\n Output:                           Nil\n \n Where:\n - `|` represents the end of the stream\n - `×` represents values that are ignored after None\n - Each Some value is processed but discarded\n - Processing stops at None\n \n ## When to Use\n - When processing Option streams for side effects until termination\n - For consuming streams with None as termination signal\n - When implementing fire-and-forget operations with early termination\n - For processing streams where None indicates completion\n - When you need to execute side effects but don&#39;t need values\n - For testing Option stream processing without result validation\n \n ## Description\n The `to_nil_none_terminated` function is a terminal operation that processes\n a stream of Options until it encounters a None value. It combines the behavior\n of `none_terminated` and `to_nil`, processing each Some value but discarding\n the results. The operation stops immediately when None is encountered, ignoring\n any subsequent elements.\n \n Key characteristics:\n - Processes Some values sequentially until None\n - Discards all values\n - Stops processing at first None\n - Ignores elements after None\n - Returns Nil regardless of input\n - Memory efficient (no accumulation)\n - Preserves side effects from stream processing\n - Terminal operation (ends stream processing)\nSynonyms:\ngs.to_nil_none_terminated\ngs to_nil_none_terminated","url":"gs.html#to_nil_none_terminated"},{"doc":"gs","title":"to_option","content":"pub fn to_option(stream: Stream(a)) -&gt; Option(a)\n Collects the first element of a stream into an option.\n \n Example:\n \n repeat(1) |&gt; take(5) |&gt; to_option\n \nSynonyms:\ngs.to_option\ngs to_option","url":"gs.html#to_option"},{"doc":"gs","title":"to_set","content":"pub fn to_set(stream: Stream(a)) -&gt; Set(a)\n Collects all elements from a stream into a set.\n \n ## Example\n \n &gt; from_list([1, 2, 2, 3, 3, 3])\n |&gt; to_set()\n |&gt; set.to_list()\n [1, 2, 3]  // Duplicates removed\n \n \n ## Visual Representation\n \n Input Stream:\n \n     +---+    +---+    +---+    +---+    +---+    +---+\n  --&gt;| 1 |---&gt;| 2 |---&gt;| 2 |---&gt;| 3 |---&gt;| 3 |---&gt;| 3 |--&gt;|\n     +---+    +---+    +---+    +---+    +---+    +---+\n       |       |        |        |        |        |\n       v       v        v        v        v        v\n      {1} -&gt; {1,2} -&gt; {1,2} -&gt; {1,2,3} -&gt; {1,2,3} -&gt; {1,2,3}\n \n Where:\n - `|` represents the end of the stream\n - Each element is added to the set, duplicates are automatically removed\n - `{...}` represents the growing set\n \n ## When to Use\n - When you need to collect unique elements from a stream\n - For removing duplicates from a stream efficiently\n - When order doesn&#39;t matter but uniqueness does\n - For implementing set operations on streams\n - When building unique collections from potentially redundant data\n - For counting unique elements in a stream\n \n ## Description\n The `to_set` function is a terminal operation that consumes the entire stream\n and collects all unique elements into a set. It automatically handles\n deduplication as elements are added to the set. The resulting set provides\n efficient membership testing and ensures each value appears only once.\n \n Key characteristics:\n - Removes duplicates automatically\n - Unordered collection (set semantics)\n - Consumes the entire stream\n - Efficient membership testing\n - Terminal operation (ends stream processing)\nSynonyms:\ngs.to_set\ngs to_set","url":"gs.html#to_set"},{"doc":"gs","title":"to_split","content":"pub fn to_split(\n  stream: Stream(a),\n  pred: fn(a) -&gt; Bool,\n) -&gt; #(Stream(a), Stream(a), Task(Nil))\n Splits a stream into two streams based on a predicate function, returning a tuple \n containing both streams and a task that manages the split operation.\n \n ## Example\n \n &gt; let #(evens, odds, task) = \n &gt;   from_range(1, 6)\n &gt;   |&gt; to_split(fn(x) { x % 2 == 0 })\n &gt; \n &gt; let even_list = evens |&gt; to_list()  // [2, 4, 6]\n &gt; let odd_list = odds |&gt; to_list()    // [1, 3, 5]\n &gt; task.await(task)\n \n \n ## Visual Representation\n \n Input:     [1]--&gt;[2]--&gt;[3]--&gt;[4]--&gt;[5]--&gt;[6]--&gt;|\n             |     |     |     |     |     |\n           pred   pred  pred  pred  pred  pred\n             |     |     |     |     |     |\n             v     v     v     v     v     v\n Evens:     ----&gt;[2]----------&gt;[4]----------&gt;[6]--&gt;|\n Odds:      [1]----------&gt;[3]----------&gt;[5]-------&gt;|\n \n Where:\n - `|` represents the end of the stream\n - `pred` represents the predicate function evaluation\n - Elements flow to either evens or odds based on predicate\n \n ## When to Use\n - When you need to partition a stream into two separate streams\n - For implementing parallel processing of different data categories\n - When separating elements based on a condition for different handling\n - For implementing filter-based routing of stream elements\n - When you need to process different categories of data independently\n - For implementing stream-based data classification\n \n ## Description\n The `to_split` function divides an input stream into two separate streams based\n on a predicate function. It returns a tuple containing:\n 1. A stream of elements for which the predicate returns true\n 2. A stream of elements for which the predicate returns false\n 3. A task that manages the splitting operation\n \n The function operates by:\n - Creating two subjects to manage the split streams\n - Asynchronously processing the input stream\n - Routing each element to the appropriate subject based on the predicate\n - Terminating both output streams when input is exhausted\n \n The splitting operation is performed lazily and asynchronously, making it\n efficient for large streams. Both resulting streams can be processed\n independently and concurrently. The task ensures proper cleanup and\n termination of both output streams.\nSynonyms:\ngs.to_split\ngs to_split","url":"gs.html#to_split"},{"doc":"gs","title":"to_subject","content":"pub fn to_subject(\n  stream: Stream(a),\n) -&gt; #(Subject(Option(a)), Task(Nil))\n Converts a stream into a process Subject and returns it along with a task that manages the conversion.\n \n ## Example\n \n &gt; let #(subject, task) = \n &gt;   from_range(1, 3)\n &gt;   |&gt; to_subject()\n &gt; \n &gt; process.receive(subject, 100)  // -&gt; Ok(Some(1))\n &gt; process.receive(subject, 100)  // -&gt; Ok(Some(2))\n &gt; process.receive(subject, 100)  // -&gt; Ok(Some(3))\n &gt; process.receive(subject, 100)  // -&gt; Ok(None)\n &gt; task.await(task)\n \n \n ## Visual Representation\n \n Stream to Subject conversion:\n \n     +---+    +---+    +---+\n  --&gt;| 1 |---&gt;| 2 |---&gt;| 3 |--&gt;|     Stream\n     +---+    +---+    +---+\n       |        |        |\n       v        v        v\n    Subject: Some(1), Some(2), Some(3), None\n       |        |        |        |\n       v        v        v        v\n    Receive  Receive  Receive  Receive     Process\n \n Where:\n - `|` represents the end of the stream\n - Each element is sent to the subject as Some(value)\n - Stream end is signaled with None\n \n ## When to Use\n - When converting from pull-based (Stream) to push-based (Subject) processing\n - For bridging synchronous streams with asynchronous message passing\n - When implementing pub/sub patterns with stream sources\n - For distributing stream elements to multiple consumers\n - When you need to buffer stream elements for later processing\n - For implementing backpressure mechanisms\n - When integrating streams with OTP process patterns\n \n ## Description\n The `to_subject` function transforms a stream into a process Subject, which\n allows for asynchronous message-based communication. It returns a tuple\n containing:\n 1. A Subject that will receive stream elements as Option(a) messages\n 2. A Task that manages the conversion process\n \n The function:\n - Creates a new Subject for receiving stream elements\n - Asynchronously processes the stream, sending each element as Some(value)\n - Signals stream completion by sending None\n - Manages the conversion process in a separate task\n - Allows for multiple consumers to receive stream elements\n \n The returned task ensures proper cleanup and should be awaited when the\n stream processing is complete. Recipients can receive values from the\n subject using `process.receive` or similar functions.\nSynonyms:\ngs.to_subject\ngs to_subject","url":"gs.html#to_subject"},{"doc":"gs","title":"try_recover","content":"pub fn try_recover(\n  stream: Stream(Result(a, b)),\n  recover: fn(b) -&gt; Stream(a),\n) -&gt; Stream(a)\n Transforms a stream of Results into a stream of values, using a recovery function for errors.\n \n ## Example\n \n from_list([Ok(1), Error(&quot;oops&quot;), Ok(3)])\n |&gt; try_recover(fn(_) { from_pure(2) })\n |&gt; to_list()\n // Returns [1, 2, 3]\n \n \n ## Visual Representation\n \n Input:   [Ok(1)]----&gt;[Err(&quot;oops&quot;)]----&gt;[Ok(3)]--&gt;|\n             |             |               |\n             v             v               v\n            [1]----&gt;  f(&quot;oops&quot;)  ------&gt;  [3]--&gt;|\n                         |\n                         v\n                        [2]\n \n Where:\n - `|` represents the end of the stream\n - `f(err)` is the recovery function\n - Errors are replaced with recovery stream values\n \n ## When to Use\n - When handling fallible operations in streams\n - For implementing retry logic in streaming operations\n - When providing fallback values for failed operations\n - For graceful error recovery in data processing pipelines\n - When implementing fault-tolerant stream processing\n - For transforming error cases into alternative valid values\n \n ## Description\n The `try_recover` function transforms a stream of Results into a stream of\n values by:\n - Passing through successful values (`Ok`) unchanged\n - Applying a recovery function to error values (`Error`)\n - Flattening the recovery stream into the main stream\n \n The recovery function takes the error value and returns a new stream,\n allowing for flexible error handling strategies including:\n - Providing default values\n - Retrying failed operations\n - Computing alternative values\n - Logging errors while continuing processing\n \n The function processes elements lazily, only applying recovery when\n needed, making it efficient for large streams.\nSynonyms:\ngs.try_recover\ngs try_recover","url":"gs.html#try_recover"},{"doc":"gs","title":"window","content":"pub fn window(stream: Stream(a), size: Int) -&gt; Stream(List(a))\n Buffers a stream into overlapping windows of a specified size.\n \n ## Example\n \n &gt; from_range(1, 5)\n |&gt; window(3)\n |&gt; to_list()\n [[1, 2, 3], [2, 3, 4], [3, 4, 5]]\n \n \n ## Visual Representation\n \n Input:     [1]--&gt;[2]--&gt;[3]--&gt;[4]--&gt;[5]--&gt;|\n             |     |     |     |     |\n             |    [1,2] [1,2,3]    |\n             |     |     |     |    |\n             |     |    [2,3,4]    |\n             |     |     |     |    |\n             |     |    [3,4,5]    |\n             \n Output: [[1,2,3], [2,3,4], [3,4,5]]\n \n Where:\n - `|` represents the end of the stream\n - Each window overlaps with the next\n - Windows slide by one element each time\n \n ## When to Use\n - For implementing sliding window operations\n - When processing time series data with overlapping windows\n - For calculating moving averages\n - When implementing signal processing algorithms\n - For pattern detection in sequential data\n - When analyzing trends in streaming data\n \n ## Description\n The `window` function creates a new stream where elements are grouped into\n overlapping windows of the specified size. Each window contains the specified\n number of consecutive elements, and windows overlap by size-1 elements. The\n function is useful for operations that need to consider multiple consecutive\n elements together.\n \n Key characteristics:\n - Creates overlapping windows of fixed size\n - Windows slide by one element each time\n - Preserves element order within windows\n - Returns empty stream if size &lt;= 0\n - Processes elements lazily\n - Memory usage proportional to window size\nSynonyms:\ngs.window\ngs window","url":"gs.html#window"},{"doc":"gs","title":"zip","content":"pub fn zip(left: Stream(a), right: Stream(b)) -&gt; Stream(#(a, b))\n Zips two streams together into a stream of tuples.\n \n ## Example\n \n &gt; from_range(1, 3)\n |&gt; zip(from_range(10, 12))\n |&gt; to_list()\n [#(1, 10), #(2, 11), #(3, 12)]\n \n \n ## Visual Representation\n \n Stream 1: [1]----&gt;[2]----&gt;[3]--&gt;|\n           |       |       |\n           v       v       v\n          #(1,10) #(2,11) #(3,12)\n           ^       ^       ^\n           |       |       |\n Stream 2: [10]---&gt;[11]--&gt;[12]--&gt;|\n \n Where:\n - `|` represents the end of the stream\n - Vertical arrows show how elements are paired\n - Each pair becomes a tuple in the output stream\n \n ## When to Use\n - When you need to combine elements from two streams pairwise\n - For implementing parallel processing with synchronized streams\n - When creating coordinate pairs from separate x and y streams\n - For matching corresponding elements from different data sources\n - When implementing stream operations that need element-wise combination\n - For creating associations between related streams\n \n ## Description\n The `zip` function combines two streams by pairing their elements together into\n tuples. It processes both streams in parallel, creating a new stream where each\n element is a tuple containing corresponding elements from both input streams.\n The resulting stream terminates when either input stream ends, making it safe\n to use with streams of different lengths.\n \n Key characteristics:\n - Preserves ordering of elements from both streams\n - Processes elements lazily (on-demand)\n - Terminates when either stream ends\n - Creates tuples of corresponding elements\n - Maintains synchronization between streams\nSynonyms:\ngs.zip\ngs zip","url":"gs.html#zip"},{"doc":"gs","title":"zip_all","content":"pub fn zip_all(\n  left: Stream(a),\n  right: Stream(b),\n) -&gt; Stream(Option(#(Option(a), Option(b))))\n Zips two streams together, continuing until both streams are exhausted and including\n `None` values for elements when one stream ends before the other.\n \n ## Example\n \n &gt; from_range(1, 3)\n |&gt; zip_all(from_range(10, 11))\n |&gt; to_list()\n [\n   Some(#(Some(1), Some(10))),\n   Some(#(Some(2), Some(11))),\n   Some(#(Some(3), None))\n ]\n \n \n ## Visual Representation\n \n Stream 1: [1]--------&gt;[2]--------&gt;[3]-------&gt;|\n           |           |           |\n           v           v           v\n        #(S(1),S(10)) #(S(2),S(11)) #(S(3),N)\n           ^           ^\n           |           |\n Stream 2: [10]-------&gt;[11]-------&gt;|\n \n Where: S = Some, N = None\n \n \n ## When to Use\n - When you need to process two streams of different lengths together\n - For implementing full outer joins between streams\n - When handling missing or incomplete data between streams\n - For synchronizing streams with potential gaps\n - When you need to know exactly where and how streams diverge\n - For implementing fault-tolerant stream processing\n \n ## Description\n The `zip_all` function combines two streams by pairing their elements, but unlike\n regular `zip` which stops at the shortest stream, this continues until both\n streams are exhausted. When one stream ends before the other, the function\n continues producing pairs with `None` for the exhausted stream&#39;s side.\n \n Key characteristics:\n - Produces values until both streams are exhausted\n - Wraps each element pair in `Some` to distinguish from stream end\n - Uses `None` to indicate when a stream has ended\n - Maintains perfect tracking of stream alignment\n - Preserves all elements from both streams\n - Processes elements lazily (on-demand)\nSynonyms:\ngs.zip_all\ngs zip_all","url":"gs.html#zip_all"},{"doc":"gs","title":"zip_all_with","content":"pub fn zip_all_with(\n  left: Stream(a),\n  right: Stream(b),\n  f: fn(Option(a), Option(b)) -&gt; c,\n) -&gt; Stream(c)\n Zips two streams using a function to combine values, continuing until both streams are exhausted.\n \n ## Example\n \n &gt; from_range(1, 3)\n |&gt; zip_all_with(\n   from_range(10, 11),\n   fn(x, y) {\n     case #(x, y) {\n       #(Some(a), Some(b)) -&gt; a + b\n       #(Some(a), None) -&gt; a\n       #(None, Some(b)) -&gt; b\n       #(None, None) -&gt; 0  // Never reached in this example\n     }\n   }\n )\n |&gt; to_list()\n // Returns [11, 13, 3]\n \n \n ## Visual Representation\n \n Stream 1: [1]--------&gt;[2]--------&gt;[3]-------&gt;|\n           |           |           |\n           v           v           v\n        f(S(1),S(10)) f(S(2),S(11)) f(S(3),N)\n           ↓           ↓             ↓\n          [11]-------&gt;[13]--------&gt;[3]-------&gt;|\n           ^           ^\n           |           |\n Stream 2: [10]-------&gt;[11]-------&gt;|\n \n Where: S = Some, N = None, f = combining function\n \n \n ## When to Use\n - When combining streams of different lengths with custom logic\n - For implementing full outer joins with transformation\n - When handling missing data with specific fallback logic\n - For stream synchronization with custom element combination\n - When implementing fault-tolerant stream processing with data transformation\n - For creating derived streams that handle gaps in source streams\n \n ## Description\n The `zip_all_with` function combines two streams by applying a custom function\n to pairs of elements, continuing until both streams are exhausted. Unlike\n regular `zip_with` which stops at the shortest stream, this continues\n processing and provides `None` values for exhausted streams.\n \n Key characteristics:\n - Processes until both streams are complete\n - Provides `None` for elements after a stream ends\n - Allows custom handling of present/missing values\n - Maintains stream alignment with transformation\n - Processes elements lazily (on-demand)\n - Preserves streaming semantics while allowing value combination\nSynonyms:\ngs.zip_all_with\ngs zip_all_with","url":"gs.html#zip_all_with"},{"doc":"gs","title":"zip_with","content":"pub fn zip_with(\n  left: Stream(a),\n  right: Stream(b),\n  f: fn(a, b) -&gt; c,\n) -&gt; Stream(c)\n Zips two streams together using a function to combine their elements.\n \n ## Example\n \n &gt; from_range(1, 3)\n |&gt; zip_with(\n   from_range(10, 12),\n   fn(x, y) { x + y }\n )\n |&gt; to_list()\n [11, 13, 15]\n \n \n ## Visual Representation\n \n Stream 1: [1]----&gt;[2]----&gt;[3]--&gt;|\n           |       |       |\n           v       v       v\n        f(1,10) f(2,11) f(3,12)\n           ^       ^       ^\n           |       |       |\n Stream 2: [10]---&gt;[11]--&gt;[12]--&gt;|\n           \n Result:   [11]---&gt;[13]--&gt;[15]--&gt;|\n \n Where:\n - `|` represents the end of the stream\n - Vertical arrows show how elements are paired\n - `f(x,y)` represents the combining function\n \n ## When to Use\n - When you need to combine elements from two streams using custom logic\n - For implementing mathematical operations between parallel streams\n - When transforming coordinate pairs with a specific formula\n - For combining time series data with a custom aggregation function\n - When implementing stream operations that need element-wise computation\n - For creating derived streams based on multiple input streams\n \n ## Description\n The `zip_with` function combines two streams by applying a custom function to\n corresponding pairs of elements. It processes both streams in parallel,\n creating a new stream where each element is the result of applying the\n provided function to elements from both input streams. The resulting stream\n terminates when either input stream ends.\n \n Key characteristics:\n - Processes elements lazily (on-demand)\n - Applies the combining function only when elements are pulled\n - Terminates when either stream ends\n - Maintains synchronization between streams\n - Allows custom transformation of paired elements\n - Preserves the streaming nature of inputs\nSynonyms:\ngs.zip_with\ngs zip_with","url":"gs.html#zip_with"},{"doc":"gs","title":"gs","content":"","url":"gs.html"},{"doc":"gs/par","title":"par_map","content":"pub fn par_map(\n  stream: Stream(a),\n  workers workers: Int,\n  with mapper: fn(a) -&gt; b,\n) -&gt; Stream(b)\n Experimental parallel map operation over a stream.\n\n ## Examples\n\n \n let numbers = gs.from_list([1, 2, 3, 4, 5])\n let doubled = \n   par_map(numbers, \n     workers: 2, \n     with: fn(x) { x * 2 }\n   )\n // -&gt; #[2, 4, 6, 8, 10]\n \n\n Warning: This is an experimental feature and should not be used in production!\nSynonyms:\ngs/par.par_map\ngs/par par_map","url":"gs/par.html#par_map"},{"doc":"gs/par","title":"gs/par","content":"","url":"gs/par.html"}]);